#ifndef __SOUND_WM8776_H
#define __SOUND_WM8776_H

/*
 *   Low-level routines for Wolfson WM8776 codec
 *
 *	Copyright (c) 2007 Rainer Zimmermann <mail@lightshed.de>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 */

struct snd_wm8776;

/* register indexes */
#define WM8776_REG_HEADPHONE_L 0x00
#define WM8776_REG_HEADPHONE_R 0x01
#define WM8776_REG_HEADPHONE_MASTER 0x02
/* digital attenuation */
#define WM8776_REG_DAC_ATTEN_L 0x03
#define WM8776_REG_DAC_ATTEN_R 0x04
#define WM8776_REG_DAC_ATTEN_MASTER 0x05
#define WM8776_REG_DAC_PHASE 0x06
#define WM8776_REG_DAC_CONTROL 0x07
#define WM8776_REG_DAC_MUTE 0x08
#define WM8776_REG_DAC_DEEMPH 0x09
#define WM8776_REG_DAC_IF_CONTROL 0x0a
#define WM8776_REG_ADC_IF_CONTROL 0x0b
#define WM8776_REG_MASTER_MODE_CONTROL 0x0c
#define WM8776_REG_POWERDOWN 0x0d
/* analogue input pga + digital attenuation */
#define WM8776_REG_ADC_ATTEN_L 0x0e
#define WM8776_REG_ADC_ATTEN_R 0x0f
#define WM8776_REG_ADC_ALC1 0x10
#define WM8776_REG_ADC_ALC2 0x11
#define WM8776_REG_ADC_ALC3 0x12
#define WM8776_REG_ADC_NOISE_GATE 0x13
#define WM8776_REG_ADC_LIMITER 0x14
#define WM8776_REG_ADC_MUX 0x15
#define WM8776_REG_OUTPUT_MUX 0x16
#define WM8776_REG_RESET 0x17

#define WM8776_NUM_REGS 0x18


/* clock ratio identifiers for snd_wm8776_set_rate() */
#define WM8776_CLOCK_RATIO_128FS 0
#define WM8776_CLOCK_RATIO_192FS 1
#define WM8776_CLOCK_RATIO_256FS 2
#define WM8776_CLOCK_RATIO_384FS 3
#define WM8776_CLOCK_RATIO_512FS 4
#define WM8776_CLOCK_RATIO_768FS 5


enum {
	WM8776_VOL_HEADPHONE_L = 0,
	WM8776_VOL_HEADPHONE_R,
	WM8776_VOL_DAC_L,
	WM8776_VOL_DAC_R,
	WM8776_VOL_ADC_GAIN_L,
	WM8776_VOL_ADC_GAIN_R,

	WM8776_NUM_VOLUMES
};

enum {
	WM8776_SW_OUT_DAC = 0,
	WM8776_SW_OUT_AUX,
	WM8776_SW_OUT_BYPASS,
	WM8776_SW_IN_1,
	WM8776_SW_IN_2,
	WM8776_SW_IN_3,
	WM8776_SW_IN_4,
	WM8776_SW_IN_5,

	WM8776_NUM_SWITCHES
};


struct snd_wm8776_ops {
	/* currently only 1 op... */
	void (*write)(void *private_data, int chip, unsigned char reg,
		      unsigned short val);
};

struct snd_wm8776 {
	struct snd_card *card;
	unsigned char chip;
	void *private_data;
	struct snd_wm8776_ops ops;
	unsigned short regs[WM8776_NUM_REGS];
	unsigned char volumes[WM8776_NUM_VOLUMES];
	unsigned int switches:WM8776_NUM_SWITCHES; /* bit field */
	struct mutex wm_mutex; /* mutex for WM8776 register access */
};

struct snd_wm8776_volume_info {
	/* number of levels (not counting mute) */
	unsigned char nlvl;
	/* declaration of dB scale (using DECLARE_TLV_DB_SCALE) */
	const unsigned int *tlv_scale;

	/*private data*/
	unsigned char reg_idx;
	unsigned short reg_mask;
	unsigned short reg_low;   /* register value for lowest level */
	unsigned short reg_mute;  /* register value for mute */
};

struct snd_wm8776_switch_info {
	unsigned char n_states;

	/*private data*/
	unsigned char reg_idx;
	unsigned char reg_inv;
	unsigned short reg_mask;
};

void snd_wm8776_reset(struct snd_wm8776 *wm, int state);
void snd_wm8776_init(struct snd_wm8776 *wm, struct snd_card *card,
			void *private_data, int chip,
			const struct snd_wm8776_ops *ops);

void snd_wm8776_reg_write(struct snd_wm8776 *wm, unsigned char reg,
			unsigned short val);
#define snd_wm8776_reg_get(wm, reg) ((wm)->regs[(reg)])

int snd_wm8776_set_volume(struct snd_wm8776 *wm, int index, int value);
int snd_wm8776_get_volume(struct snd_wm8776 *wm, int index);
const struct snd_wm8776_volume_info *snd_wm8776_get_volume_info(int index);
int snd_wm8776_set_switch(struct snd_wm8776 *wm, int index, int value);
int snd_wm8776_get_switch(struct snd_wm8776 *wm, int index);
const struct snd_wm8776_switch_info *snd_wm8776_get_switch_info(int index);
void snd_wm8776_set_rate(struct snd_wm8776 *wm, unsigned char clock_ratio);


#endif /* __SOUND_WM8776_H */
